@@ -7,7 +7,7 @@ use crate::core::UniqueButton;
 
 /// Button definition, it's simply a hashmap, but is used to represent all the components of the button
 #[derive(Serialize, Deserialize, Debug, Clone, Default)]
-pub struct Button(HashMap<String, Value>);
+pub struct Button(pub HashMap<String, Value>);
 
 impl Button {
     /// Creates a new empty button
